import { Coordinate, Layer } from './mapvis.types';
import { layerTypes } from './components/layers';
import { Layer as DeckLayer } from '@deck.gl/core/typed';

export const getProperty = (obj: any, accessorString: string): any => {
  const accessors = accessorString.split('.');
  let value = obj;
  for (const accessor of accessors) {
    value = value[accessor];
    if (value === undefined) {
      break;
    }
  }
  return value;
};

export const makeLayer = (type: string): Layer => {
  const layerType = layerTypes[type] as any;

  if (!layerType) {
    throw new Error(`Invalid layer type: ${type}`);
  }

  return {
    id: Date.now(),
    name: 'New layer',
    type: layerType,
    config: {
      ...layerType.layerOptions,
    },
    visible: true,
  };
};

export const getLocationInfo = async (search: string) => {
  try {
    const query: string = encodeURIComponent(search);
    const url: string = `https://nominatim.openstreetmap.org/search?q=${query}&format=json`;
    const response = await fetch(url);
    const data = await response.json();

    if (data && data.length > 0) {
      const { boundingbox } = data[0];
      return boundingbox.map(parseFloat);
    } else {
      return null;
    }
  } catch (e) {
    return null;
  }
};

export const getDistance = (loc1: Coordinate, loc2: Coordinate): number => {
  const [minLon, minLat]: Coordinate = loc1;
  const [maxLon, maxLat]: Coordinate = loc2;
  const lonDistance: number = (maxLon ?? 0) - (minLon ?? 0);
  const latDistance: number = (maxLat ?? 0) - (minLat ?? 0);
  const featureHypot: number = Math.hypot(lonDistance, latDistance);
  return featureHypot;
};
